﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using PCIELM4_MC; 

namespace PCIELM4_SingleAxis
{
    public partial class Form1 : Form
    {
        const ushort BoardNo = 0;
        const int AxisQty = 2;
        
        public Form1()
        {
            InitializeComponent();
            EnableUI(false); 
        }

        private void btnConnect_Click(object sender, EventArgs e)
        {
            MC_RETURN eRet;
            int i = 0;
            byte AxisNo = 0;

            //-------------------------------
            //STEP 1: Open Driver
            //-------------------------------
            eRet = CLM4MC.pcielm4_open_driver();
            if (eRet != MC_RETURN.SUCCESS)
	        {
	            MessageBox.Show("Open PCIELM4 driver failed!");
                timer1.Enabled = false;
                EnableUI(false);
                return;
                
	        }

            //-------------------------------
            //STEP 2: Check PCIELM4 is in PCIe Slot
            //-------------------------------
	        eRet =  CLM4MC.pcielm4_check_card_in_slot( BoardNo );
	        if(eRet != MC_RETURN.SUCCESS)
	        {
                MessageBox.Show("PCIELM4 board 0 was not detected!");
                EnableUI(false);
		        return;
	        }
	        
	        //-------------------------------
	        //STEP 3: Intialize PCIELM4 card
	        //-------------------------------
	        eRet =  CLM4MC.pcielm4_registration( BoardNo );
	        
            for (i = 0; i < AxisQty; i++)
            {
                AxisNo = (byte)(0x01 << i);

                // PulseMode = 0: OUT/DIR, 1: CW/CCW, 2: A/B Phase
                eRet = CLM4MC.pcielm4_set_pls_cfg(BoardNo, AxisNo, 2, 0, 0);

                // EncoderMode = 1: CW/CCW, 2~4: A/B Phase 
                eRet = CLM4MC.pcielm4_set_enc_cfg(BoardNo, AxisNo, 2);

                // StopMode = 1: slow down then stop, 2: sudden stop
                // RefSource = 1: encoder counter, 2: command counter
                eRet = CLM4MC.pcielm4_set_limit(BoardNo, AxisNo, 0, 2);
                //eRet = CLM4MC.pcielm4_set_softlimit(BoardNo, AxisNo, 1, 2, 0x7FFFFFFF, -0x7FFFFFFF);
                eRet = CLM4MC.pcielm4_set_softlimit_disable(BoardNo, AxisNo);

                eRet = CLM4MC.pcielm4_set_home_cfg(BoardNo, AxisNo, 0, 0, 0, 0x55, 0);
                eRet = CLM4MC.pcielm4_set_alarm(BoardNo, AxisNo, 0, 0);
                eRet = CLM4MC.pcielm4_set_inp(BoardNo, AxisNo, 0, 0);
                eRet = CLM4MC.pcielm4_set_ready(BoardNo, AxisNo, 0, 0);
                eRet = CLM4MC.pcielm4_set_latch_cfg(BoardNo, AxisNo, 0, 0);
                eRet = CLM4MC.pcielm4_disable_vring_counter( BoardNo, AxisNo);
            }
            EnableUI(true);
            timer1.Enabled = true;


        }

        private void btnDisconnect_Click(object sender, EventArgs e)
        {
            timer1.Enabled = false;
            CLM4MC.pcielm4_close_driver();
            EnableUI(false);
            
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            MC_RETURN eRet = 0;
            int i = 0;
            byte AxisNo = 0;

            ushort IOStatus = 0;
            int LogicPos = 0;
            int EncPos = 0;
            int Speed = 0;
            byte Done = 0;
            ushort StopStatus = 0;

            TextBox[] txtIOStatus = { txtIOStatus0, txtIOStatus1 };
            TextBox[] txtCmdCnt = { txtCmdCnt0, txtCmdCnt1 };
            TextBox[] txtEncCnt = { txtEncCnt0, txtEncCnt1 };
            TextBox[] txtCurSpeed = { txtCurSpeed0, txtCurSpeed1 };
            TextBox[] txtMotionStatus = { txtMotionStatus0, txtMotionStatus1 };

            for (i = 0; i < AxisQty; i++)
            {
                AxisNo = (byte)(0x01 << i);

                eRet = CLM4MC.pcielm4_get_mdi_status(BoardNo, AxisNo, ref IOStatus);
                eRet = CLM4MC.pcielm4_get_cmdcounter(BoardNo, AxisNo, ref LogicPos);
                eRet = CLM4MC.pcielm4_get_enccounter(BoardNo, AxisNo, ref EncPos);
                eRet = CLM4MC.pcielm4_get_speed(BoardNo, AxisNo, ref Speed);
                eRet = CLM4MC.pcielm4_get_motion_done(BoardNo, AxisNo, ref Done, ref StopStatus);


                txtIOStatus[i].Text = "0x" + IOStatus.ToString("X4");
                txtCmdCnt[i].Text = LogicPos.ToString();
                txtEncCnt[i].Text = EncPos.ToString();
                txtCurSpeed[i].Text = Speed.ToString();

                if (Done == 0)
                { txtMotionStatus[i].Text = "Running"; }
                else
                { txtMotionStatus[i].Text = "Done"; }
            }
        }

        private void ResetCnt(object sender, EventArgs e)
        {
            int index = int.Parse((sender as Button).Tag.ToString());

            int i = 0;
            MC_RETURN eRet = 0;
            byte AxisNo = 0;
            int LogicPos = 0;
            int EncPos = 0;

            if (index < AxisQty)
            {
                AxisNo = (byte)(0x01 << index);
                eRet = CLM4MC.pcielm4_set_cmdcounter(BoardNo, AxisNo, LogicPos);
                eRet = CLM4MC.pcielm4_set_enccounter(BoardNo, AxisNo, EncPos);
            }
            else if (index == AxisQty)
            {
                for (i = 0; i < AxisQty; i++)
                {
                    AxisNo = (byte)(0x01 << i);
                    eRet = CLM4MC.pcielm4_set_cmdcounter(BoardNo, AxisNo, LogicPos);
                    eRet = CLM4MC.pcielm4_set_enccounter(BoardNo, AxisNo, EncPos);
                }
            } 
        }

        private void ToggleSVON(object sender, EventArgs e)
        {
            int index = int.Parse((sender as Control).Tag.ToString());

            byte AxisNo = (byte)(0x01 << index);
            MC_RETURN eRet = 0;
            byte OnOff = 0;

            eRet = CLM4MC.pcielm4_get_servo_on_status(BoardNo, AxisNo, ref OnOff);
            if (OnOff == 0)
            { eRet = CLM4MC.pcielm4_set_servo_on(BoardNo, AxisNo, 1); }
            else
            { eRet = CLM4MC.pcielm4_set_servo_on(BoardNo, AxisNo, 0); }
        }

        private void ToggleRALM(object sender, EventArgs e)
        {
            int index = int.Parse((sender as Control).Tag.ToString());

            byte AxisNo = (byte)(0x01 << index);
            MC_RETURN eRet = 0;
            byte OnOff = 0;

            eRet = CLM4MC.pcielm4_get_servo_almrst_status(BoardNo, AxisNo, ref OnOff);
            if (OnOff == 0)
            { eRet = CLM4MC.pcielm4_set_alarm_reset(BoardNo, AxisNo, 1); }
            else
            { eRet = CLM4MC.pcielm4_set_alarm_reset(BoardNo, AxisNo, 0); }
        }

        
        private void MoveAxis(object sender, EventArgs e)
        {
            int index = int.Parse((sender as Control).Tag.ToString());

            TextBox[] txtSpeed = { txtSpeed0, txtSpeed1 };
            TextBox[] txtAcc = { txtAcc0, txtAcc1 };
            TextBox[] txtDist = { txtDist0, txtDist1};

            byte AxisNo = (byte)(0x01 << index);
            MC_RETURN eRet = 0;

            uint Speed = System.UInt32.Parse(txtSpeed[index].Text);
            uint Acc = System.UInt32.Parse(txtAcc[index].Text);
            int Dist = System.Int32.Parse(txtDist[index].Text);

            eRet = CLM4MC.pcielm4_t_move(BoardNo, AxisNo, Speed / 10, Speed, Speed / 10, Acc, Acc, Dist);
        }

        private void StopAxis(object sender, EventArgs e)
        {
            int index = int.Parse((sender as Control).Tag.ToString());

            int i = 0;
            byte AxisNo = 0;
            MC_RETURN eRet = 0;

            if (index < AxisQty)
            {
                AxisNo = (byte)(0x01 << index);
                // StopMode = 1: slow down then stop, 2: sudden stop
                eRet = CLM4MC.pcielm4_stop_move(BoardNo, AxisNo, 1);
            }
            else if (index == AxisQty)
            {
                for (i = 0; i < AxisQty; i++)
                {
                    AxisNo = (byte)(0x01 << i);
                    // StopMode = 1: slow down then stop, 2: sudden stop
                    eRet = CLM4MC.pcielm4_stop_move(BoardNo, AxisNo, 1);
                }
            }
        }

        // Enable/Disable User Interface:
        public void EnableUI(bool enable) 
        {
            if(enable)
            {
                btnConnect.Enabled = false;
                btnDisconnect.Enabled = true;
                btnResetCnt0.Enabled = true;
                btnResetCnt1.Enabled = true;
                btnResetCntAll.Enabled = true;
                btnSVON0.Enabled = true;
                btnSVON1.Enabled = true;
                btnRalm0.Enabled = true;
                btnRalm1.Enabled = true;

                btnMove0.Enabled = true;
                btnMove1.Enabled = true;
                btnStop0.Enabled = true;
                btnStop1.Enabled = true;
                btnStopAll.Enabled = true;
            
            }
            else
            {
                btnConnect.Enabled = true;
                btnDisconnect.Enabled = false;
                btnResetCnt0.Enabled = false;
                btnResetCnt1.Enabled = false;
                btnResetCntAll.Enabled = false;
                btnSVON0.Enabled = false;
                btnSVON1.Enabled = false;
                btnRalm0.Enabled = false;
                btnRalm1.Enabled = false;

                btnMove0.Enabled = false;
                btnMove1.Enabled = false;
                btnStop0.Enabled = false;
                btnStop1.Enabled = false;
                btnStopAll.Enabled = false;
            }
        
        }

    }
}
